#!/usr/bin/env python
'''Trains a simple convnet on the MNIST dataset.

Gets to 99.25% test accuracy after 12 epochs
(there is still a lot of margin for parameter tuning).
16 seconds per epoch on a GRID K520 GPU.
'''

from __future__ import print_function
import numpy as np
np.random.seed(1337)  # for reproducibility

#from keras.datasets import mnist
# import necessary modules
from keras.models import Sequential
from keras.layers import Dense, Dropout, Activation, Flatten
from keras.layers import Convolution2D, MaxPooling2D
from keras.utils import np_utils
from keras import backend as K

from pre_post import *

#(X_train, y_train), (X_test, y_test) = cPickle.load(f)
#f = gzip.open('/project/fchen14/csc7700/data/mnist_expanded.pkl.gz', 'rb')
#train_set, valid_set, test_set = cPickle.load(f)

# load the mnist dataset
import cPickle
import gzip
f = gzip.open('mnist.pkl.gz', 'rb')
#f = gzip.open('mnist_expanded.pkl.gz', 'rb')
# load the training and test dataset
(X_train, y_train), (X_test, y_test) = cPickle.load(f)
print(X_train.shape, y_train.shape, X_test.shape, y_test.shape)

preview_train(X_train, y_train)

import sys

batch_size = 128
nb_classes = 10
nb_epoch = 50
#nb_fc_neurons = int(sys.argv[1])

# input image dimensions
img_rows, img_cols = 28, 28
# number of convolutional filters to use
nb_filters = 32
# size of pooling area for max pooling
pool_size = (2, 2)
# convolution kernel size
kernel_size = (3, 3)

# the data, shuffled and split between train and test sets
#(X_train, y_train), (X_test, y_test) = mnist.load_data()

#if K.image_dim_ordering() == 'th':
#    X_train = X_train.reshape(X_train.shape[0], 1, img_rows, img_cols)
#    X_test = X_test.reshape(X_test.shape[0], 1, img_rows, img_cols)
#    input_shape = (1, img_rows, img_cols)
#else:
#    X_train = X_train.reshape(X_train.shape[0], img_rows, img_cols, 1)
#    X_test = X_test.reshape(X_test.shape[0], img_rows, img_cols, 1)
#    input_shape = (img_rows, img_cols, 1)

if K.image_dim_ordering() == 'th':
    X_train = X_train.reshape(X_train.shape[0],  img_rows*img_cols)
    X_test = X_test.reshape(X_test.shape[0],  img_rows*img_cols)
    input_shape = (img_rows*img_cols,)
else:
    X_train = X_train.reshape(X_train.shape[0], img_rows*img_cols)
    X_test = X_test.reshape(X_test.shape[0], img_rows*img_cols)
    input_shape = (img_rows*img_cols,)

X_train = X_train.astype('float32')
X_test = X_test.astype('float32')
X_train /= 255
X_test /= 255
print('X_train shape:', X_train.shape)
print('X_test shape:', X_test.shape)
print(X_train.shape[0], 'train samples')
print(X_test.shape[0], 'test samples')

# convert class vectors to binary class matrices
Y_train = np_utils.to_categorical(y_train, nb_classes)
Y_test = np_utils.to_categorical(y_test, nb_classes)

model = Sequential()

act_func='relu'
#act_func='sigmoid'
#act_func='tanh'
#model.add(Dense(nb_classes,activation='softmax',input_shape=input_shape))
pdropout=0.25
model.add(Dense(200,activation=act_func,input_shape=input_shape))
model.add(Dropout(pdropout))
model.add(Dense(100,activation=act_func))
model.add(Dropout(pdropout))
model.add(Dense( 60,activation=act_func))
model.add(Dropout(pdropout))
model.add(Dense( 30,activation=act_func))
model.add(Dropout(pdropout))
model.add(Dense(nb_classes,activation='softmax'))

model.compile(loss='categorical_crossentropy',
              optimizer='adadelta',
              metrics=['accuracy'])

from dp3 import DynamicUpdate
history=DynamicUpdate()

h = model.fit(X_train, Y_train, batch_size=batch_size, nb_epoch=nb_epoch,
          verbose=1,callbacks=[history], validation_data=(X_test, Y_test))
score = model.evaluate(X_test, Y_test, verbose=0)

print('Test score:', score[0])
print('Test accuracy:', score[1])

pt = h.history['acc']
pv = h.history['val_acc']
print(np.max(pt), np.max(pv), np.argmax(pv))

visu_results(X_test, y_test, model)
#print('using nb_fc_neurons=',)
#nls = np.array(h)
#np.savetxt("loss_history.txt", nls, delimiter = ",", fmt='%1.4e')
raw_input('press...')
